@@ -45,45 +45,50 @@ class SungrowModbusWebClient(BaseModbusClient):
 
         self.dev_host = host
         self.ws_port = port
-        self.ws_socket = None
         self.timeout = kwargs.get('timeout',  '5')
+        self.ws_socket = None
         BaseModbusClient.__init__(self, framer(ClientDecoder(), self), **kwargs)
         
         self.ws_endpoint = "ws://" + str(self.dev_host) + ":" + str(self.ws_port) + "/ws/home/overview"
         self.ws_token = ""
         self.dev_type = ""
-        self.dev_procotol = ""
         self.dev_code = ""
 
     def connect(self):
         """ Connect via WebSocket to Dongle, retrieve Device details and Token
         :returns: True if connection succeeded, False otherwise
         """
-        
+
         if self.ws_token:
             return True
-        
-        self.ws_socket = create_connection(self.ws_endpoint)
+
+        try:
+            self.ws_socket = create_connection(self.ws_endpoint,timeout=self.timeout)
+        except Exception as err:
+            logging.debug(f"Connection to websocket server failed: {self.ws_endpoint}, Message: {err}")
+            return None
+
         logging.debug("Connection to websocket server established: " + self.ws_endpoint)
         
         self.ws_socket.send(json.dumps({ "lang": "en_us", "token": self.ws_token, "service": "connect" }))
         try:
             result =  self.ws_socket.recv()
-        except:
+        except Exception as err:
             result =  ""
-            logging.error("Connection Failed")
-            raise ConnectionException(self.__str__())
+            raise ConnectionException(f"Websocket error: {str(err)}")
 
-        payload_dict = json.loads(result)
-        logging.debug(payload_dict)
+        try:
+            payload_dict = json.loads(result)
+            logging.debug(payload_dict)
+        except Exception as err:
+            raise ConnectionException(f"Data error: {str(result)}\n\t\t\t\t{str(err)}")       
         
         if payload_dict['result_msg'] == 'success':
             self.ws_token = payload_dict['result_data']['token']
             logging.info("Token Retrieved: " + self.ws_token)
         else:
             self.ws_token = ""
-            logging.error("Connection Failed", payload_dict['result_msg'] )
-            raise ConnectionException(self.__str__())
+            raise ConnectionException(f"Connection Failed {payload_dict['result_msg']}")
         
         logging.debug("Requesting Device Information")
         self.ws_socket.send(json.dumps({ "lang": "en_us", "token": self.ws_token, "service": "runtime" }))
@@ -111,6 +116,9 @@ class SungrowModbusWebClient(BaseModbusClient):
         :param request: The encoded request to send
         :return: The number of bytes written
         """
+
+        if not self.ws_token:
+            self.connect()
         
         self.header = request
         
@@ -126,33 +134,40 @@ class SungrowModbusWebClient(BaseModbusClient):
         address = (256*request[8]+request[9]) + 1
         count = 256*request[10]+request[11]
         dev_id = str(request[6])
+        self.payload_modbus = ""
         
         logging.debug("param_type: " + str(param_type) + ", start_address: " + str(address) + ", count: " + str(count) + ", dev_id: " +str(dev_id))       
-        
         logging.debug(f'Calling: http://{str(self.dev_host)}/device/getParam?dev_id={dev_id}&dev_type={str(self.dev_type)}&dev_code={str(self.dev_code)}&type=3&param_addr={address}&param_num={count}&param_type={str(param_type)}&token={self.ws_token}&lang=en_us&time123456={str(int(time.time()))}')
-        r =requests.get(f'http://{str(self.dev_host)}/device/getParam?dev_id={dev_id}&dev_type={str(self.dev_type)}&dev_code={str(self.dev_code)}&type=3&param_addr={address}&param_num={count}&param_type={str(param_type)}&token={self.ws_token}&lang=en_us&time123456={str(int(time.time()))}')
+        try:
+            r =requests.get(f'http://{str(self.dev_host)}/device/getParam?dev_id={dev_id}&dev_type={str(self.dev_type)}&dev_code={str(self.dev_code)}&type=3&param_addr={address}&param_num={count}&param_type={str(param_type)}&token={self.ws_token}&lang=en_us&time123456={str(int(time.time()))}', timeout=self.timeout)
+        except Exception as err:
+            raise ConnectionException(f"HTTP Request failed: {str(err)}")
+        logging.debug("HTTP Status code " + str(r.status_code))
         if str(r.status_code) == '200':
-            logging.debug("Connection Sucsessful: Status code - " + str(r.status_code))
             self.payload_dict = json.loads(str(r.text))
+            logging.debug("Payload Status code " + str(self.payload_dict.get('result_code', "N/A")))
             logging.debug("Payload Dict: " + str(self.payload_dict))
-            modbus_data = self.payload_dict['result_data']['param_value'].split(' ')
-            modbus_data.pop() # remove null on the end
-            data_len = int(len(modbus_data)) # length of data
-            logging.debug("Data length: " + str(data_len))
-            # Build the Header, The header is consumed by pyModbus and not actually sent to the device
-            # [aaaa][bbbb][cccc][dd][ee][ff] a = Transaction ID, b = Protocol ID, c = Message Length (Data + Header of 3), d =  Device ID, e = Function Code, f = Data Length
-            self.payload_modbus = ['00', format(request[1], '02x'), '00', '00', '00', format((data_len+3), '02x'), format(request[6], '02x'), format(request[7], '02x'), format(data_len, '02x')]
-            # Attach the data we recieved from HTTP request to the header we created to make a modbus RTU message
-            self.payload_modbus.extend(modbus_data)
-            logging.debug("Modbus Header: " + str(self.payload_modbus))
-            logging.debug("Modbus Data: " + str(modbus_data))
-            #logging.debug("Modbus RTU: " + str(self.payload_modbus))
-            return self.payload_modbus
-
+            if self.payload_dict.get('result_code',0) == 1:
+                modbus_data = self.payload_dict['result_data']['param_value'].split(' ')
+                modbus_data.pop() # remove null on the end
+                data_len = int(len(modbus_data)) # length of data
+                logging.debug("Data length: " + str(data_len))
+                # Build the Header, The header is consumed by pyModbus and not actually sent to the device
+                # [aaaa][bbbb][cccc][dd][ee][ff] a = Transaction ID, b = Protocol ID, c = Message Length (Data + Header of 3), d =  Device ID, e = Function Code, f = Data Length
+                self.payload_modbus = ['00', format(request[1], '02x'), '00', '00', '00', format((data_len+3), '02x'), format(request[6], '02x'), format(request[7], '02x'), format(data_len, '02x')]
+                # Attach the data we recieved from HTTP request to the header we created to make a modbus RTU message
+                self.payload_modbus.extend(modbus_data)
+#                logging.debug("Modbus Header: " + str(self.payload_modbus))
+#                logging.debug("Modbus Data: " + str(modbus_data))
+                #logging.debug("Modbus RTU: " + str(self.payload_modbus))
+                return self.payload_modbus
+            elif self.payload_dict.get('result_code',0) == 106:
+                self.ws_token = ""
+                raise ConnectionException(f"Token Expired: {str(self.payload_dict.get('result_code'))}:{str(self.payload_dict.get('result_msg'))} ")
+            else:
+                raise ConnectionException(f"Connection Failed: {str(self.payload_dict.get('result_code'))}:{str(self.payload_dict.get('result_msg'))} ")
         else:
-            logging.warning("Connection Failed: Status code - " + str(r.status_code))
-            self.ws_token = ""
-            return None
+            raise ConnectionException(f"Connection Failed: {str(self.payload_dict.get('result_code'))}:{str(self.payload_dict.get('result_msg'))} ")
 
     def _recv(self, size):
         """ This doesn't actually recieve any data.
@@ -160,6 +175,7 @@ class SungrowModbusWebClient(BaseModbusClient):
         """
 
         if not self.payload_modbus:
+            logging.error(f"Recieve Failed: payload is empty")
             raise ConnectionException(self.__str__())
 
         # If size isn't specified read up to 4096 bytes at a time.
@@ -187,7 +203,7 @@ class SungrowModbusWebClient(BaseModbusClient):
         del self.payload_modbus[0:counter]
 
         logging.debug("Requested Size: " + str(size) + ", Returned Size: " + str(counter))
-        logging.debug("Returned Data: " + str(data))
+#        logging.debug("Returned Data: " + str(data))
 
         if  int(counter) < int(size):
             return self._handle_abrupt_socket_close(size, data, time.time() - time_)
